/*
 * DodgeDefenderPanel.java Max Biel
 *
 * CapStone Project
 *
 * Copyright (c) 2013 Max Biel. All Rights Reserved.
 *
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;
import java.net.*;
import java.applet.*;

/**
 * Intro screen for Dodge Defender. Includes logo and music. Press space-bar to start game.
 *
 * @version 1.10 13 Apr 2013
 * @author Max Biel
 *
 * history: !!!
 *      xxx 13apr2013: started class
 */
public class IntroScreen extends JPanel
{ 
    private Image logo;
    private ArrayList<Enemy> enemies = new ArrayList<Enemy>();
    private int RADIUS = 6;
    private AudioClip clip;
    
    /**
     * Constructor for IntroScreen. Loads all instance variables and starts listeners, as well as
     * adds first wave of enemies and sets up our hero.
     */
    public IntroScreen()
    {
        loadImage();
        playMusic();
        KeyListener1 keyListener = new KeyListener1();
        addKeyListener(keyListener);
        
        for (int i = 0; i < 10; i++)
        {
            Enemy enemy = new Enemy();
            enemies.add(enemy);
        }
        
        TimerListener timerListener = new TimerListener();
        Timer timer = new Timer(10, timerListener);
        setBackground(Color.BLACK);
        timer.start();
        setFocusable(true);
    }
    
    /**
     * This is what is called when repaint() is called on the panel.
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        int x;
        int y;
        for (int i = 0; i < enemies.size(); i++)
        {
            x = enemies.get(i).getX();
            y = enemies.get(i).getY();
            
            g.setColor(Color.GREEN);
            
            g.fillOval(x, y, RADIUS, RADIUS);
        }
        
        g.drawImage(logo, 90, 225, this);
        g.setColor(Color.WHITE);
        g.drawString("Press the SpaceBar to Start!", 310, 400);
        g.drawString("Game by: Max Biel", 675, 690);
    }
    
    /**
     * Loads the intro image.
     */
    public void loadImage()
    {
        URL url = getClass().getResource("dodgeLogo.jpg");
        logo = getToolkit().getImage(url);
    }
    
    /**
     * Plays initial music.
     */
    public void playMusic()
    {
        URL url = getClass().getResource("8-bit.wav");
        clip = Applet.newAudioClip(url);
        clip.play();
    }
    
    /**
     * Listens for key press to switch panels.
     */
    private class KeyListener1 implements KeyListener
    {
        /**
         * If key is spacebar, starts the main game from the applet and stops the music.
         */
        public void keyPressed (KeyEvent e)
        {
            if (e.getKeyCode() == KeyEvent.VK_SPACE)
            {
                System.out.println("Key was pressed");
                DodgeDefender applet = (DodgeDefender)getTopLevelAncestor();
                clip.stop();
                applet.mainGame();
            }
        }
        
        /**
         * Not used.
         */
        public void keyReleased (KeyEvent e)
        {
        }
        
        /**
         * Not used.
         */
        public void keyTyped (KeyEvent e)
        {
        }
    }
    
    /**
     * Listener that fires every x milliseconds from the timer.
     */
    private class TimerListener implements ActionListener
    {
        /**
         * Puts enemies in background and wraps them (just for show).
         */
        public void actionPerformed(ActionEvent e)
        {
            for (int i = 0; i < enemies.size(); i++)
            {
                enemies.get(i).wrap();
                enemies.get(i).move();
                repaint();
                requestFocusInWindow();
            }
        }
    }
}