/*
 * DodgeDefenderPanel.java Max Biel
 *
 * CapStone Project
 *
 * Copyright (c) 2013 Max Biel. All Rights Reserved.
 *
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;
import java.net.*;
import java.applet.*;
import java.util.Random;

/**
 * Drawing panel for DodgeDefender applet.
 *
 * @version 1.10 08 Apr 2013
 * @author Max Biel
 *
 * history: !!!
 *      xxx 08apr2013: started class
 */
public class MainGame extends JPanel
{
    private final int HERO_RADIUS = 6;
    private final int THRUSTER_RADIUS1 = 2;
    private final int THRUSTER_RADIUS2 = 4;
    
    private ArrayList<Enemy> enemies = new ArrayList<Enemy>();
    private Hero hero;
    private int waveNum;
    private Graphics g;
    private URL gameOverURL;
    private Image gameOver;
    private URL oneURL;
    private URL twoURL;
    private URL threeURL;
    private Image one;
    private Image two;
    private Image three;
    private AudioClip thruster;
    private URL thrusterURL;
    private AudioClip explosion;
    private URL explosionURL;
    private Random generator = new Random();
    private ArrayList<Image> delayImages = new ArrayList<Image>();
    private Timer timer;
    private Timer frameTempo;
    
    /**
     * Constructor for the MainGame class. Sets instance variables and creates first enemies and hero.
     * Sets wave number and background color. Sets up all listeners and timers.
     */
    public MainGame()
    {
        for (int i = 0; i < 10; i++)
        {
            if (i < 2)
            {
                Enemy enemy = new Enemy(15);
                enemies.add(enemy);
            }
            
            else
            {
                Enemy enemy = new Enemy();
                enemies.add(enemy);
            }
        }
        
        hero = new Hero();
        
        gameOverURL = getClass().getResource("game_over.jpg");
        gameOver = getToolkit().getImage(gameOverURL);
        
        oneURL = getClass().getResource("1.jpg");
        twoURL = getClass().getResource("2.jpg");
        threeURL = getClass().getResource("3.jpg");
        
        one = getToolkit().getImage(oneURL);
        two = getToolkit().getImage(twoURL);
        three = getToolkit().getImage(threeURL);
        
        delayImages.add(one);
        delayImages.add(two);
        delayImages.add(three);
        
        KeyListener1 keyListener = new KeyListener1();
        
        FrameRate frameRate = new FrameRate();
        TimerListener timerListener = new TimerListener();
        
        frameTempo = new Timer(10, frameRate);
        timer = new Timer(50, timerListener);
        
        frameTempo.start();
        timer.start();
        
        addKeyListener (keyListener);
        thrusterURL = getClass().getResource("thruster.wav");
        thruster = Applet.newAudioClip(thrusterURL);
        explosionURL = getClass().getResource("explode.wav");
        explosion = Applet.newAudioClip(explosionURL);
        
        setFocusable(true);

        waveNum = 1;
        
        setBackground (Color.BLACK);
        
    }
    
    /**
     * This is called when repaint() is called on the panel.
     */
    public void paintComponent (Graphics page)
    {

        super.paintComponent (page);
        
        g = page;      
        g.setColor(Color.WHITE);
        
        g.drawString("Level: " + waveNum, 5, 15);
        
        if (waveNum == 1)
        {
            g.drawString("Directions: Get to the right side of the screen using the arrow keys!", 200, 675);
        }
        drawEnemies();
        if (hero.getAlive() == true)
        {
            drawHero();
        }
        
        else
        {
            g.drawImage(gameOver, 200, 300, this);
            g.setColor(Color.RED);
            g.drawString("Press \"R\" to Restart!", 350, 375);
        }
    }
    
    /**
     * Redraws all enemies based on their x,y coordinates. 
     */
    public void drawEnemies()
    {
        int x;
        int y;
        int radius;
        for (int i = 0; i < enemies.size(); i++)
        {
            x = enemies.get(i).getX();
            y = enemies.get(i).getY();
            radius = enemies.get(i).getRadius();
            
            g.setColor(Color.GREEN);
            
            g.fillOval(x, y, radius , radius);
        }
    }
    
    /**
     * Redraws hero based on his/her x,y cooridinate.
     */
    public void drawHero()
    {
        int x = hero.getX();
        int y = hero.getY();
        
        g.setColor(Color.RED);
        
        g.fillOval(x, y, HERO_RADIUS, HERO_RADIUS);
    }

    /**
     * Draws 10 more enemies. Resets all positions. Increases wave number.
     */
    public void levelUp()
    {
                    
        for (int i = 0; i < enemies.size(); i++)
        {
            enemies.get(i).setX(799);
        }
        
        for (int i = 0; i < 10; i++)
        { 
            if (waveNum > 10)
            {
                Enemy enemy = new Enemy(generator.nextInt(5) + 25);
            }
            
            if (waveNum > 7)
            {
                Enemy enemy = new Enemy(generator.nextInt(5) + 20);
                enemies.add(enemy);
            }
            
            else if (waveNum > 5)
            {
                Enemy enemy = new Enemy(generator.nextInt(5) + 15);
                enemies.add(enemy);
            }
            
            else if (waveNum > 1)
            {
                Enemy enemy = new Enemy(generator.nextInt(5) + 10);
                enemies.add(enemy);
            }
            
            else
            {
                if (i < 2)
                {
                    Enemy enemy = new Enemy(15);
                    enemies.add(enemy);
                }
                
                else
                {
                    Enemy enemy = new Enemy();
                    enemies.add(enemy);
                }
            }
            
            
           
        }
        
        waveNum ++;
        
        hero.setX(1);
        hero.setY(400);
        hero.setSpeedX(0);
        hero.setSpeedY(0);
    }
    
    /**
     * Removes hero from screen and plays explosion sound.
     */
    public void gameOver()
    {
        hero.die();
    }
     
        
    /**
     * Moves hero based on which key is pressed.
     */
    private class KeyListener1 implements KeyListener
    {
        /**
         * Increases/Decreases x/y speed based on key presses, and plays sound no matter which key is pressed.
         * If 'R' is pressed, bring user back to intro screen to restart game.
         */
        public void keyPressed (KeyEvent e)
        {
            thruster.play();
            if (e.getKeyCode() == KeyEvent.VK_UP)
            {
                hero.setSpeedY(hero.getSpeedY() - 1);
            }
            else if (e.getKeyCode() == KeyEvent.VK_DOWN)
            {
                hero.setSpeedY(hero.getSpeedY() + 1);
            }
            
            else if (e.getKeyCode() == KeyEvent.VK_RIGHT)
            {
                hero.setSpeedX(hero.getSpeedX() + 1);
            }
            
            else if (e.getKeyCode() == KeyEvent.VK_LEFT)
            {
                hero.setSpeedX(hero.getSpeedX() - 1);
            }
            
            else if (e.getKeyCode() == KeyEvent.VK_R)
            {
                DodgeDefender applet = (DodgeDefender)getTopLevelAncestor();
                thruster.stop();
                applet.introScreen();
            }
        }
        
        /**
         * Stops sound when key is released.
         */
        public void keyReleased (KeyEvent e)
        {
            thruster.stop();
        }
        
        /**
         * Not used.
         */
        public void keyTyped (KeyEvent e)
        {
        }
    }
    
    /**
     * Mantains framerate for smoothness (allows game-tempo to be slow but doesn't slow framerate).
     */
    private class FrameRate implements ActionListener
    {
        /**
         * Repaints screen every x milliseconds according to javax timer.
         */
        public void actionPerformed(ActionEvent e)
        {

            repaint();
            requestFocusInWindow();

        }
    }
    
    /**
     * Moves characters around screen and detects collisions.
     */
    private class TimerListener implements ActionListener
    {
        /**
         * Moves characters, detects collisions, advances level.
         */
        public void actionPerformed(ActionEvent e)
        {
            for (int i = 0; i < enemies.size(); i++)
            {
                int enemyX = enemies.get(i).getX();
                int enemyY = enemies.get(i).getY();
                double dist = Math.sqrt((Math.pow(hero.getX() - enemyX, 2)) + (Math.pow(hero.getY() - enemyY, 2)));
                
                enemies.get(i).wrap();
                enemies.get(i).move();
                
                if (dist < HERO_RADIUS * 2)
                {
                    if (hero.getAlive() == true)
                    {
                        explosion.play();
                    }
                    gameOver();
                }
            }
            
            hero.move();
            hero.wrap();
            
            if (hero.getX() > 800)
            {
                levelUp();
            }
        }
    }
                
                
    
    
    
}